#include "Arduino.h"
#include "Tasker.h"

void CheckButtonA();
void CheckButtonB();
void StartGas();
void StopGas();
void StartSpark();
void StopSpark();
void SetBlink();
void StartWelder();
void StopWelder();





Tasker tasker;

// constants won't change. They're used here to
// set pin numbers:
const int btnA = 2;    		// FootSwitch Button
const int btnB = 3;    		// Function Button
const int ledPWR = 13;      // Power LED
const int ledPWR2 = 7;      // Power LED
const int pinGas = 8;      	// Gas Pin
const int pinWelder = 9;      	// Gas Pin
const int pinSpark = 5; 	// Spark Pin

const int gasBeforeTime = 800;
const int gasAfterTime = 5000;
const int sparkTime = 250;

const int ledIntervalFn = 250;

// Variables will change:
int statePwrLed = HIGH;
int stateFn = HIGH;
int stateGas = LOW;
int stateSpark = LOW;

int cntBtnA = 0;
int cntBtnB = 0;

void setup() {
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(btnA, INPUT);
  pinMode(btnB, INPUT);
  pinMode(ledPWR, OUTPUT);
  pinMode(ledPWR2, OUTPUT);
  pinMode(pinGas, OUTPUT);
  pinMode(pinSpark, OUTPUT);
  pinMode(pinWelder, OUTPUT);



  // set initial Output states
  digitalWrite(ledPWR, stateFn);
  digitalWrite(ledPWR2, stateFn);
  digitalWrite(pinGas, LOW);
  digitalWrite(pinSpark, LOW);
  digitalWrite(pinWelder, HIGH);



  Serial.begin(38400);
  Serial.println ("----------   SYSTEM START  ----------");

  tasker.setInterval(CheckButtonA, 10);
  tasker.setInterval(CheckButtonB, 200);

}

void loop() {

	tasker.loop();


}

void CheckButtonA() {

	int btnDebounce = 10;

	if(cntBtnA < 0) cntBtnA = 0;
	if(cntBtnA >= btnDebounce) cntBtnA = btnDebounce;

	if(digitalRead(btnA)) {
		cntBtnA ++;

		if(cntBtnA == btnDebounce) { //here is state change


			if(stateFn == HIGH) { // TIG or MMA (LOW)
				StartGas();
				StartWelder();
				//Serial.println (tasker.scheduledIn(StopGas));
				if(tasker.scheduledIn(StopGas) > 0) { //Re-ignition (no pre-gas when Gas is already running
					StartSpark();
				} else {
					tasker.setTimeout(StartSpark, gasBeforeTime); //Normal ignition with pre-gas
				}
				tasker.cancel(StopGas); // for preventing to stop gas when pressed twice
			} else {
				digitalWrite(pinSpark, HIGH);
			}


		}

	} else {
		cntBtnA --;

		if(cntBtnA == 0) {
			if(stateFn == HIGH) { // TIG or MMA (LOW)
				StopWelder();
				tasker.setTimeout(StopGas, gasAfterTime);
				tasker.cancel(StartSpark); // prevents sparking after very short press
			} else {
				digitalWrite(pinSpark, LOW);
			}
		}
	}
}

void CheckButtonB() {

  digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));

	if(digitalRead(btnB)) {

		tasker.setInterval(CheckButtonB, 30);

		cntBtnB ++;

		if(cntBtnB == 10) {
			stateFn = !stateFn;
			//Serial.println (cntBtnB);
			SetBlink();

		}

	} else {
		cntBtnB = 0;
		tasker.setInterval(CheckButtonB, 200);
	}
}

void StartGas() {
	digitalWrite(pinGas, HIGH);
	stateGas = HIGH;
}
void StopGas() {
	digitalWrite(pinGas, LOW);
	stateGas = LOW;
}
void StartSpark() {
	digitalWrite(pinSpark, HIGH);
	stateSpark = HIGH;
	tasker.setTimeout(StopSpark, sparkTime);
}
void StopSpark() {
	digitalWrite(pinSpark, LOW);
	stateSpark = LOW;
}

void StartWelder() {
	digitalWrite(pinWelder, LOW);
}

void StopWelder() {
	digitalWrite(pinWelder, HIGH);
}

void SetBlink() {

	if(stateFn == HIGH) {
		statePwrLed = HIGH;
		tasker.cancel(SetBlink);
	} else {
		statePwrLed = !statePwrLed;
		tasker.setInterval(SetBlink, ledIntervalFn);
	}

	digitalWrite(ledPWR, statePwrLed);
	digitalWrite(ledPWR2, statePwrLed);

}
