#include "Arduino.h"
#include "Tasker.h"

#define TIG_MODE HIGH
#define MMA_MODE LOW
#define PRINT_STATES LOW  //set to HIGH to print actions (GAS STARTED etc...)

void CheckButtonState_btnTrigger();
void CheckButtonState_btnMode();
void StartGas();
void StopGas();
void StartIgnition();
void StopIgnition();
void SetBlinkInterval();
void StartWelder();
void StopWelder();

Tasker tasker;

// constants won't change. They're used here to
// set pin numbers:
const int btnTrigger = 2;    		// FootSwitch Button
const int btnMode = 3;    		// Function Button
const int ledPWR = 13;      // Power LED (Board Built in LED)
const int ledPWR2 = 7;      // Power LED
const int pinGas = 8;      	// Gas Pin
const int pinWelder = 9;      	// Gas Pin
const int pinIgnition = 5; 	// Spark Pin

//Times in milliseconds
const int gasBeforeTime = 1000;
const int gasAfterTime = 4000;
const int sparkTime = 500;

// Variables will change:
int statePwrLed = HIGH;
int mode = TIG_MODE;
int stateGas = LOW;
int stateSpark = LOW;

int cntBtnTrigger = 0;
int cntBtnMode = 0;

void setup() {
 
  pinMode(btnTrigger, INPUT);
  pinMode(btnMode, INPUT);
  pinMode(ledPWR, OUTPUT);
  pinMode(ledPWR2, OUTPUT);
  pinMode(pinGas, OUTPUT);
  pinMode(pinIgnition, OUTPUT);
  pinMode(pinWelder, OUTPUT);

  // set initial Output states
  digitalWrite(ledPWR, mode);
  digitalWrite(ledPWR2, mode);
  digitalWrite(pinGas, LOW);
  digitalWrite(pinIgnition, LOW);
  digitalWrite(pinWelder, HIGH);

  Serial.begin(9600);
  Serial.println ("----------   SYSTEM START  ----------");

  //Run CheckButtonState_btnTrigger() function every 10 millieconds
  tasker.setInterval(CheckButtonState_btnTrigger, 10); 
  
   //Run CheckButtonState_btnMode() function every 200 millieconds
  tasker.setInterval(CheckButtonState_btnMode, 200); 

  //start blinking power LEDs
  tasker.setInterval(SetBlinkInterval, 650);
  Serial.println("Program Start");
  mode = TIG_MODE;

}

void loop() {

	tasker.loop();

}


//periodicaly called every 10ms
void CheckButtonState_btnTrigger() {

	int btnDebounce = 10;

	if(cntBtnTrigger < 0) cntBtnTrigger = 0;
	if(cntBtnTrigger > btnDebounce) cntBtnTrigger = btnDebounce;

	if(digitalRead(btnTrigger)) { //button is pressed
		
		cntBtnTrigger ++;           //wait 150ms.....

    	if(cntBtnTrigger == btnDebounce) { //...and start ignition, welder and gas

    		Serial.println(cntBtnTrigger);
    		if(mode == TIG_MODE) {
				StartGas();
				StartWelder();
				if(tasker.scheduledIn(StopGas) > 0) { //Re-ignition (no pre-gas when Gas is already running)
					StartIgnition();
				} else {
					tasker.setTimeout(StartIgnition, gasBeforeTime); //Normal ignition with pre-gas
				}
				tasker.cancel(StopGas); // for preventing to stop gas when pressed twice
			} else { //MMA_MODE
				if(PRINT_STATES) Serial.println ("IGNITION START (MMA)");
				digitalWrite(pinIgnition, HIGH); //in MMA mode just starts ignition
			}
		}

	} else { //Trigger button is released

		cntBtnTrigger --; // wait...

		if(cntBtnTrigger == 0 && (stateGas == HIGH || mode == MMA_MODE)) { //.. and stop welder and gas (AND chcek if trigger was really pressed in TIG mode)
			Serial.println("cntBtnTrigger = 0");
			if(mode == TIG_MODE) { 
				StopWelder();
				tasker.setTimeout(StopGas, gasAfterTime);
			} else { //MMA_MMODE

				digitalWrite(pinIgnition, LOW);
			}
		}
	}
}

//periodicaly called every 200ms
void CheckButtonState_btnMode() {

	if(digitalRead(btnMode)) {

		tasker.setInterval(CheckButtonState_btnMode, 30); //changes calling interval to 30ms

		cntBtnMode ++; // wait 10 x 30ms....

		if(cntBtnMode == 10) { //... and change mode
			mode = !mode; //Change mode
			//Serial.println (cntBtnB);
			SetBlinkInterval();//change pwrLEDs blinkink interval
			if(mode == TIG_MODE){
				if(PRINT_STATES) Serial.println("mode = TIG_MODE");
			} else {
				if(PRINT_STATES) Serial.println("mode = MMA_MODE");
			}
		}

	} else {
		cntBtnMode = 0;
		tasker.setInterval(CheckButtonState_btnMode, 200);
	}
}

void StartGas() {
	if(PRINT_STATES) Serial.println ("GAS START");
	digitalWrite(pinGas, HIGH);
	stateGas = HIGH;
}
void StopGas() {
	if(PRINT_STATES) Serial.println ("GAS STOP");
	digitalWrite(pinGas, LOW);
	stateGas = LOW;
}
void StartIgnition() {
	if(PRINT_STATES) Serial.println ("IGNITION START");
  digitalWrite(pinIgnition, HIGH);
	stateSpark = HIGH;
	tasker.setTimeout(StopIgnition, sparkTime);
}
void StopIgnition() {
	if(PRINT_STATES) Serial.println ("IGNITION STOP");
	digitalWrite(pinIgnition, LOW);
	stateSpark = LOW;
}

void StartWelder() {
	if(PRINT_STATES) Serial.println ("WELDER START");
	digitalWrite(pinWelder, LOW);
}

void StopWelder() {
	if(PRINT_STATES) Serial.println ("WELDER STOP");
	digitalWrite(pinWelder, HIGH);
}

void SetBlinkInterval() {

	if(mode == HIGH) {
		tasker.setInterval(SetBlinkInterval, 650);
	} else {
		tasker.setInterval(SetBlinkInterval, 150);
	}

  statePwrLed = !statePwrLed;
	digitalWrite(ledPWR, statePwrLed);
	digitalWrite(ledPWR2, statePwrLed);
  
  


}
